Scriptname TStoryMCM extends SKI_ConfigBase  
{Toy Story MCM and Config}

;.......... Version
Int Function GetVersion()
	Return 0010 ;reminder update ToysVersionRequired 
EndFunction
String Function TStoryVersion()
	Return "v0.91"
EndFunction
Bool Function TStoryToysVersionCorrect()
	Int ToysVersionRequired = 0029
	If Main.Toys.VersionNum() >= ToysVersionRequired
		Return True
	Else
		Main.TStoryLog("ERROR: Incorrect version of Toys installed", True)
		Return False
	EndIf
EndFunction

;========== Initialize
Event OnInit()
	Parent.OnInit()
	Utility.WaitMenuMode(1)
	Main.RegisterForSingleUpdateGameTime(0.03)
EndEvent
;.......... Runs on new game
Event OnConfigInit() 
	Main.TStoryLog("New Game Init", True)
	Main.DelayedInit = True
	TStoryInitialize()
EndEvent 
;.......... Each Game Re-Load & New Game
;also called from Player Alias
Function TStoryInitialize()
	Main.PeriodicFirstCount = 0
	Main.DelayedLoadInit = True
	Main.TStoryLog("Load Game Init - Toy Story " + TStoryVersion() + " initialized", True)
	Debug.Notification("Toy Story " + TStoryVersion())
	Main.RegisterForSingleUpdateGameTime(0.03)
EndFunction

;.......... Upgrading 
Event OnVersionUpdate(Int a_version)
	Utility.WaitMenuMode(1)
	Main.TStoryLog("Upgrading to " + TStoryVersion(), True)
	Debug.Notification("Toy Story is upgrading to " + TStoryVersion())
	TStoryInitPages()
	TStoryInitArrays()
	Main.UnregisterForUpdateGameTime()
	Main.RegisterForSingleUpdateGameTime(0.03)
	;If CurrentVersion < 1000 
		;do stuff
	;EndIf
EndEvent

;.......... Initialize Pages
Function TStoryInitPages()
 	Pages = new string[2] 
 	Pages[0] = " Settings"
	Pages[1] = " Run Tests"
EndFunction

;.......... Initialize Menu Arrays
Function TStoryInitArrays()

EndFunction

;.......... Page Selection & Setup
Event OnPageReset(string page)
 	If (page == "") 
 		LoadCustomContent("TStory/ToyStoryLogo.dds", 27, 21) 
		Return 
	Else 
 		UnloadCustomContent() 
 	EndIf 
	If (page == " Settings")
		TStorySettingsPage()
	ElseIf (page == " Run Tests")
		TStoryTestPage()
	Endif
EndEvent

;.......... Settings Page
Function TStorySettingsPage()
	SetCursorFillMode(TOP_TO_BOTTOM)
	AddHeaderOption("Toy Story " + TStoryVersion() + " by The Toys Team")
	
	AddEmptyOption()
	ExportToggleID = AddTextOption("Save MCM Settings", "Save")
	ImportToggleID = AddTextOption("Load MCM Settings", "Load")

	AddEmptyOption()
	VerboseLogID = AddToggleOption("Verbose Log", TStoryVerboseLog)
	TipsID = AddToggleOption("Pop-up Tip Messages", TStoryTips)
	
	AddEmptyOption()
	AddHeaderOption("Dev Modes")
	DevTipsID = AddToggleOption("Pop-up Dev Tip Messages", TStoryDevTips)
	ModEventsID = AddToggleOption("Mod Event Notifications", TStoryModEvents)
	
	SetCursorPosition(1)
	AddHeaderOption("Curse of Talos")
	AddSliderOptionST("RandomToyChanceSlider", "Chance of Random Toy", TStoryRandomToyChance)
	PreferenceID = AddToggleOption("Toy Box Preference", TStoryPreference)
EndFunction

;.......... Test Page
Function TStoryTestPage()
	SetCursorFillMode(TOP_TO_BOTTOM)

	AddHeaderOption("GetToy() & HandleToy()")
	Test1ID = AddTextOption("Equip random pelvis from any box", "Run")
	Test3ID = AddTextOption("Random Equip from Insignia box", "Run")
	Test4ID = AddTextOption("Random Equip from ANY box", "Run")
	
	AddHeaderOption("GetToy()")
	Test8ID = AddTextOption("Random select toys from ANY box", "Run")
	
	AddHeaderOption("ReserveSlots()")
	Test5ID = AddTextOption("Reserve Slots", "Run")
	Test6ID = AddTextOption("Un-Reserve Slots", "Run")
	
	AddHeaderOption("IsSlotsReserved()")
	Test7ID = AddTextOption("Are Slots Reserved?", "Run")
	
	SetCursorPosition(1)
	AddHeaderOption("UnequipToys()")
	Test2ID = AddTextOption("Unequip pelvis", "Run")
	Test9ID = AddTextOption("Unequip All excelpt Pelvis", "Run")
	Test10ID = AddTextOption("Unequip All to Storage", "Run")
	
	AddHeaderOption("UnStoreToys()")
	Test11ID = AddTextOption("Un-store", "Run")

	AddHeaderOption("HideToys() / UnHideToys()")
	Test14ID = AddTextOption("Hide Toys", "Run")
	Test15ID = AddTextOption("UnHide Toys", "Run")
	
	AddHeaderOption("IsSlotAvailable()")
	Test12ID = AddTextOption("Can Pelvis Slot be used?", "Run")
	Test13ID = AddTextOption("Can Pelvis Slot be used by storing?", "Run")
EndFunction	

Event OnOptionSelect(int option)
	If (option == ExportToggleID)
		If ShowMessage("SAVE... Are you sure? This will replace any previous save.")
			TStoryExportSettings()	
			Debug.MessageBox("Saved")		
		Endif
	ElseIf (option == ImportToggleID)
		If ShowMessage("LOAD... Are you sure? This will replace all existing settings with your last save.")
			TStoryImportSettings()
			ForcePageReset()
			Debug.MessageBox("Loaded")		
		Endif
	ElseIf (option == TipsID)
		TStoryTips = !TStoryTips
		SetToggleOptionValue(TipsID, TStoryTips)
	ElseIf (option == DevTipsID)
		TStoryDevTips = !TStoryDevTips
		SetToggleOptionValue(DevTipsID, TStoryDevTips)
	ElseIf (option == ModEventsID)
		TStoryModEvents = !TStoryModEvents
		SetToggleOptionValue(ModEventsID, TStoryModEvents)
		If TStoryModEvents
			Main.TStoryRegModEvents()
		Else
			Main.TStoryUnRegModEvents()
		EndIf
	ElseIf (option == VerboseLogID)
		TStoryVerboseLog = !TStoryVerboseLog
		SetToggleOptionValue(VerboseLogID, TStoryVerboseLog)
	ElseIf (option == PreferenceID)
		TStoryPreference = !TStoryPreference
		SetToggleOptionValue(PreferenceID, TStoryPreference)		
	ElseIf (option == Test1ID)
		Main.TStoryTest1()
	ElseIf (option == Test2ID)
		Main.TStoryTest2()
	ElseIf (option == Test3ID)
		Main.TStoryTest3()
	ElseIf (option == Test4ID)
		Main.TStoryTest4()
	ElseIf (option == Test5ID)
		Main.TStoryTest5()
	ElseIf (option == Test6ID)
		Main.TStoryTest6()
	ElseIf (option == Test7ID)
		Main.TStoryTest7()
	ElseIf (option == Test8ID)
		Main.TStoryTest8()
	ElseIf (option == Test9ID)
		Main.TStoryTest9()
	ElseIf (option == Test10ID)
		Main.TStoryTest10()
	ElseIf (option == Test11ID)
		Main.TStoryTest11()
	ElseIf (option == Test12ID)
		Main.TStoryTest12()
	ElseIf (option == Test13ID)
		Main.TStoryTest13()
	ElseIf (option == Test14ID)
		Main.TStoryTest14()
	ElseIf (option == Test15ID)
		Main.TStoryTest15()
	Endif
EndEvent

Event OnOptionHighlight(int option)	
	If (option == TipsID)
		SetInfoText("[ON] Tips Messages are shown. Recommended for first time users.")
	ElseIf (option == DevTipsID)
		SetInfoText("[ON] Dev Tips Messages are shown. These tips are for mod authors and display when interactions occur with Toys,  giving a reference for within Toy Story's script.")
	ElseIf (option == ModEventsID)
		SetInfoText("[ON] Notificattions are shown when Mod Events fire. Is for mod authors.")
	ElseIf (option == VerboseLogID)
		SetInfoText("[ON] Detailed logging. Required if reporting a bug. [OFF] Only critical log entries.")
	ElseIf (option == PreferenceID)
		SetInfoText("[ON] Preference is given to the 'most worn toy box'. For example, if the majority of toys already worn are from the Insignia box, then random toys added will be from that same box if possible. [OFF] Toys equipped are completely random from any box.")
	ElseIf (option == Test2ID)
		SetInfoText("Unequip pelvis if possible")
	ElseIf (option == Test3ID)
		SetInfoText("Random Equip from Insignia box where possible")
	ElseIf (option == Test4ID)
		SetInfoText("Random Equip from ANY box where possible")
	ElseIf (option == Test5ID)
		SetInfoText("Reserve Slots if possible")
	ElseIf (option == Test6ID)
		SetInfoText("Un-Reserve Slots")
	ElseIf (option == Test7ID)
		SetInfoText("Are Slots Reserved by another mod?")
	ElseIf (option == Test8ID)
		SetInfoText("Random select toys from ANY box")
	ElseIf (option == Test9ID)
		SetInfoText("Unequip All excelpt Pelvis where possible, and destroy")
	ElseIf (option == Test10ID)
		SetInfoText("Unequip All to Storage where possible. Fails if slots not reserved")
	ElseIf (option == Test11ID) 
		SetInfoText("Un-store")
	ElseIf (option == Test12ID) 
		SetInfoText("Can Pelvis Slot be used?")
	ElseIf (option == Test13ID) 
		SetInfoText("Can Pelvis Slot be used by storing?")
	ElseIf (option == Test14ID) 
		SetInfoText("Hide all toys inappropriate for animation scenes")
	ElseIf (option == Test15ID) 
		SetInfoText("UnHide the hidden toys")
	EndIf
EndEvent

;......... Menu Based Options
Event OnOptionMenuOpen(int option)
    ;if (option == AnalDeviceID)
    ;    SetMenuDialogOptions(AnalDeviceList)
    ;    SetMenuDialogStartIndex(TStoryAnalDeviceSelect)
    ;    SetMenuDialogDefaultIndex(0)
	;EndIf
EndEvent

State RandomToyChanceSlider
	Event OnSliderOpenST()
		SetSliderDialogStartValue(TStoryRandomToyChance)
		SetSliderDialogDefaultValue(50)
		SetSliderDialogRange(0, 100)
		SetSliderDialogInterval(10)
	EndEvent
	Event OnSliderAcceptST(float value)
		TStoryRandomToyChance = value as Int
		SetSliderOptionValueST(TStoryRandomToyChance)
	EndEvent
	Event OnHighlightST()
		SetInfoText("[0 to Disable] Chance of Talos Priests (there are 3) equipping a random toy when they have line of sight with you.  They won't replace toys from another mod if they are restricted. You can avoid their gaze by sneaking.")
	EndEvent
EndState

;.......... Import / Export
function ExportInt(string Name, int Value)
	JsonUtil.SetIntValue(File, Name, Value)
endFunction

int function ImportInt(string Name, int Value)
	return JsonUtil.GetIntValue(File, Name, Value)
endFunction

function ExportBool(string Name, bool Value)
	JsonUtil.SetIntValue(File, Name, Value as int)
endFunction

bool function ImportBool(string Name, bool Value)
	return JsonUtil.GetIntValue(File, Name, Value as int) as bool
endFunction

function ExportFloat(string Name, float Value)
	JsonUtil.SetFloatValue(File, Name, Value)
endFunction

float function ImportFloat(string Name, float Value)
	return JsonUtil.GetFloatValue(File, Name, Value)
endFunction

function TStoryExportSettings()
	File = "../TStory/TStoryMCM.json"
	JsonUtil.SetStringValue(File, "ExportLabel", Main.PlayerRef.GetLeveledActorBase().GetName()+" - "+Utility.GetCurrentRealTime() as int)
	JsonUtil.SetStringValue(File, "Version", TStoryVersion())	

	ExportBool("TStoryTips", TStoryTips)
	ExportBool("TStoryDevTips", TStoryDevTips)
	ExportBool("TStoryModEvents", TStoryModEvents)
	ExportBool("TStoryVerboseLog", TStoryVerboseLog)
	ExportBool("TStoryPreference", TStoryPreference)
	
	ExportInt("TStoryRandomToyChance", TStoryRandomToyChance)
	JsonUtil.Save(File, false)
endFunction

function TStoryImportSettings()
	File = "../TStory/TStoryMCM.json"

	TStoryTips = ImportBool("TStoryTips", TStoryTips)
	TStoryDevTips = ImportBool("TStoryDevTips", TStoryDevTips)
	TStoryModEvents = ImportBool("TStoryModEvents", TStoryModEvents)
	TStoryVerboseLog = ImportBool("TStoryVerboseLog", TStoryVerboseLog)
	TStoryPreference = ImportBool("TStoryPreference", TStoryPreference)
	
	TStoryRandomToyChance = ImportInt("TStoryRandomToyChance", TStoryRandomToyChance)	
endFunction

;========== Properties  ==========
;=================================

;.......... Tests
Int Test1ID
Int Test2ID
Int Test3ID
Int Test4ID
Int Test5ID
Int Test6ID
Int Test7ID
Int Test8ID
Int Test9ID
Int Test10ID
Int Test11ID
Int Test12ID
Int Test13ID
Int Test14ID
Int Test15ID

;.......... Toggles
Int InstallTestToggleID
Int ExportToggleID
Int ImportToggleID
Int TipsID
Int DevTipsID
Int ModEventsID
Int VerboseLogID
Int PreferenceID

TStoryMain Property Main Auto
Bool Property TStoryTips = False Auto
Bool Property TStoryDevTips = False Auto
Bool Property TStoryModEvents = False Auto
Bool Property TStoryVerboseLog = False Auto
Bool Property TStoryPreference = True Auto
Int Property TStoryRandomToyChance =50 Auto
String File